package com.lqkj.link.module.bulletin.controller;

import com.lqkj.link.message.MessageBean;
import com.lqkj.link.message.MessageListBean;
import com.lqkj.link.module.bulletin.domain.NoticeInfo;
import com.lqkj.link.module.bulletin.service.NoticeInfoService;
import com.lqkj.link.module.jwt.service.JwtService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import static com.lqkj.link.APIVersion.VERSION_V1;

@RestController
@RequestMapping("/notice")
@Tag(name = "通知公告接口", description = "通知公告接口")
public class NoticeInfoController {
    private final NoticeInfoService noticeInfoService;
    private final JwtService jwtService;

    public NoticeInfoController(NoticeInfoService noticeInfoService, JwtService jwtService) {
        this.noticeInfoService = noticeInfoService;
        this.jwtService = jwtService;
    }

    @Operation(
            summary = "5.1.2.14 我的通知",
            description = "5.1.2.14 我的通知"
    )
    @PostMapping("/" + VERSION_V1 + "/myNotice")
    public MessageListBean<NoticeInfo> myNotice(HttpServletRequest request) {
        String authHeader = request.getHeader("Authorization");
        String userCode = jwtService.decryptUsernameWithHeader(authHeader);
        return MessageListBean.ok(noticeInfoService.queryList(userCode), "我的通知");
    }

    @Operation(
            summary = "5.1.2.15 查看通知",
            description = "5.1.2.15 查看通知"
    )
    @PostMapping("/" + VERSION_V1 + "/view")
    public MessageBean<String> view(@RequestParam Integer infoId) {
        noticeInfoService.view(infoId);
        return MessageBean.ok(null, "查看通知");
    }
}
