package com.lqkj.link.module.encrypt.controller;

import com.lqkj.link.message.MessageBean;
import com.lqkj.link.util.RSAUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/encrypt")
@Tag(name = "加密工具类", description = "加密工具类")
public class EncryptController {

    @Operation(
            summary = "5.1.1.1 获取RSA公钥接口",
            description = "5.1.1.1 获取RSA公钥接口"
    )
    @PostMapping("/rsaKey")
    public MessageBean<String> rsaKey() {
        return MessageBean.ok(RSAUtils.generateBase64PublicKey(), "获取RSA公钥");
    }

    @Operation(
            summary = "5.1.1.5 RSA加密接口",
            description = "5.1.1.5 RSA加密接口",
            parameters = {
                @Parameter(name = "word", description = "需加密的字符串", required = true)
            }
    )
    @PostMapping("/rsaEncrypt")
    public MessageBean<String> rsaEncrypt(String word) {
        return MessageBean.ok(RSAUtils.encryptBase64(word), "RSA加密数据");
    }

    @Operation(
            summary = "5.1.1.6 RSA解密接口",
            description = "5.1.1.6 RSA解密接口",
            parameters = {
                    @Parameter(name = "word", description = "需解密的字符串", required = true)
            }
    )
    @PostMapping("/rsaDecrypt")
    public MessageBean<String> rsaDecrypt(String word) {
        return MessageBean.ok(RSAUtils.decryptBase64(word), "RSA解密数据");
    }
}
