package com.lqkj.link.module.zone.repository;

import com.lqkj.link.module.zone.domain.LayerInfo;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface LayerInfoRepository extends JpaRepository<LayerInfo, String> {

    @Modifying
    @Query(nativeQuery = true,
        value = "with t1 as(insert into layer_info(layer_id, zone_id, layer_name) " +
                    "select nextval('layer_info_layer_id_seq'), :zoneId, layer_name " +
                    "from layer_info where zone_id = :templateId  order by layer_id returning *)" +
                "select min(t1.layer_id) - min(li.layer_id) from t1, layer_info li where li.zone_id = :templateId"
    )
    Integer createWithTemplateReturnIncrement(@Param("templateId") Integer templateId,
                                              @Param("zoneId") Integer zoneId);

    @Modifying
    @Query(nativeQuery = true,
            value = "insert into layer_info(layer_id, zone_id, layer_name) " +
                    "select nextval('layer_info_layer_id_seq'), :zoneId, layer_name " +
                    "from layer_info where zone_id = :templateId  order by layer_id returning *"
    )
    List<LayerInfo> createWithTemplate(@Param("templateId") Integer templateId,
                                              @Param("zoneId") Integer zoneId);

    List<LayerInfo> queryByZoneIdOrderByLayerId(Integer zoneId);

    List<LayerInfo> queryAllByZoneId(Integer zoneId);

    @Query(nativeQuery = true,
        value = "with t1 as (select layer_id, bool_and(locking) as locking from geom_info where zone_id = :zoneId and layer_id is not null group by layer_id) " +
                "select li.layer_id as \"layerId\", li.layer_name as \"layerName\", coalesce(t1.locking, false) as locking, concat(li.layer_id) as \"Name\" " +
                "from layer_info li left join t1 on li.layer_id = t1.layer_id"
    )
    List<Map<String, Object>> queryAllByZoneIdV2(Integer zoneId);

    @Query(nativeQuery = true,
        value = "select count(*) > 0 from layer_info where zone_id = :zoneId"
    )
    Boolean hasLayer(@Param("zoneId") Integer zoneId);
}
