package com.lqkj.link.module.zone.repository;

import com.lqkj.link.module.zone.domain.ZoneInfo;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface ZoneInfoRepository extends JpaRepository<ZoneInfo, Integer> {

    @Query(nativeQuery = true,
        value = "select * from zone_info " +
                "where user_id is null " +
                    "and (:zoneName = '' or zone_name like concat('%', :zoneName, '%')) " +
                "order by update_time desc"
    )
    Page<ZoneInfo> pageQueryTemplate(@Param("zoneName") String zoneName,
                             Pageable pageable);

    @Query(nativeQuery = true,
        value = "select distinct(zone_name) as \"name\" from zone_info zi, user_info ui " +
                "where zi.user_id = ui.user_id and ui.user_code = :userCode and zone_name like concat('%', :keyword, '%') " +
                "order by zone_name"
    )
    List<Map<String, Object>> autoCompleteMyWork(@Param("userCode") String userCode,
                                                 @Param("keyword") String keyword);

    @Query(nativeQuery = true,
        value = "select distinct (zone_name) as \"name\" from zone_info " +
                "where zone_name like concat('%', :keyword, '%') " +
                    "and (user_id is null or auth_status = 2) " +
                    "and (:searchType = 0 or (:searchType = 1 and user_id is null) or (:searchType = 2 and user_id is not null)) " +
                "order by zone_name"
    )
    List<Map<String, Object>> autoCompleteResource(@Param("keyword") String keyword,
                                                   @Param("searchType") Integer searchType);

    @Query(nativeQuery = true,
            value = "select distinct (zone_name) as \"name\" from zone_info zi, audit_record ar " +
                    "where ar.zone_id = zi.zone_id and ar.auth_status != 4 and ar.auth_status != 5 " +
                    "and zone_name like concat('%', :keyword, '%') " +
                    "and (:searchType = 0 or (:searchType = 1 and ar.auth_status = 1 ) or (:searchType = 2 and ar.auth_status != 1)) " +
                    "order by zone_name"
    )
    List<Map<String, Object>> autoCompleteAudit(@Param("keyword") String keyword,
                                                   @Param("searchType") Integer searchType);

    @Query(nativeQuery = true,
        value = "select zi.* from zone_info zi, user_info ui " +
                "where zi.user_id = ui.user_id and ui.user_code = :userCode " +
                    "and zi.zone_name like concat('%', :name, '%') " +
                "order by zi.update_time desc"
    )
    Page<ZoneInfo> pageQueryMyWork(@Param("userCode") String userCode,
                                   @Param("name") String name,
                                   Pageable pageable);

    @Query(nativeQuery = true,
        value = "select zi.zone_id as \"zoneId\", zi.zone_name as \"zoneName\"," +
                "zi.thumbnail, ui.head_img as \"headImg\", ui.display_name as \"displayName\"," +
                "zi.share_count as \"shareCount\", zi.view_count as \"viewCount\", " +
                "ui.user_id as \"authorId\", zi.like_count as \"likeCount\" " +
                "from zone_info zi left join user_info ui " +
                "on zi.user_id = ui.user_id " +
                "where zone_name like concat('%', :name, '%') " +
                "and (zi.user_id is null or zi.auth_status = 2) " +
                "and (:searchType = 0 or (:searchType = 1 and zi.user_id is null) or (:searchType = 2 and zi.user_id is not null)) " +
                "order by " +
                "CASE WHEN :orderType = 0 THEN zi.update_time END DESC," +
                "CASE WHEN :orderType = 1 THEN zi.like_count END DESC," +
                "CASE WHEN :orderType = 2 THEN zi.view_count END DESC;"
    )
    Page<Map<String, Object>> pageQueryResourceCenter(@Param("name") String name,
                                           @Param("searchType") Integer searchType,
                                           Pageable pageable,
                                           @Param("orderType") Integer orderType);

    @Modifying
    @Query(nativeQuery = true,
        value = "with t1 as (update zone_info set share_count = coalesce(share_count, 0) + 1 where zone_id = :zoneId)" +
                "insert into share_info(record_id, zone_id, user_id, share_code, share_time, can_use, shared_user_id) " +
                "select nextval('share_info_record_id_seq'), :zoneId, user_id, :shareCode, now(), 7 , :sharedUserId " +
                "from user_info where user_code = :userCode"
    )
    void share(@Param("zoneId") Integer zoneId,
               @Param("userCode") String userCode,
               @Param("shareCode") String shareCode,
               @Param("sharedUserId") Integer sharedUserId);

    @Modifying
    @Query(nativeQuery = true,
            value = "with t1 as (update zone_info set view_count = coalesce(view_count, 0) + 1 where zone_id = :zoneId)" +
                    "insert into view_record(record_id, zone_id, user_id, create_time) " +
                    "select nextval('view_record_record_id_seq'), :zoneId, user_id, now() " +
                    "from user_info where user_code = :userCode"
    )
    void view(@Param("zoneId") Integer zoneId,
               @Param("userCode") String userCode);

    @Query(nativeQuery = true,
        value = "select count(*) > 0 from zone_info zi, user_info ui " +
                "where zi.zone_id != :zoneId and zi.zone_name = :zoneName " +
                    "and zi.user_id = ui.user_id and ui.user_code = :userCode"
    )
    Boolean hasSameNameWithUser(@Param("zoneName") String zoneName,
                                @Param("userCode") String userCode,
                                @Param("zoneId") Integer zoneId);

    Boolean existsByZoneNameAndUserId(String zoneName, Integer userId);

    @Modifying
    @Query(nativeQuery = true,
        value = "update zone_info set zone_name = :zoneName, update_time = now() where zone_id = :zoneId"
    )
    void rename(@Param("zoneName") String zoneName,
                @Param("zoneId") Integer zoneId);

    @Query(nativeQuery = true,
        value = "select ui.user_code = :userCode from zone_info zi, user_info ui " +
                "where zi.zone_id = :zoneId and zi.user_id = ui.user_id "
    )
    Boolean isAuthor(@Param("zoneId") Integer zoneId,
                     @Param("userCode") String userCode);

    @Query(nativeQuery = true,
        value = "select auth_status != 1 from zone_info where zone_id = :zoneId"
    )
    Boolean canBeDeleteWithAuthStatus(@Param("zoneId") Integer zoneId);

    @Query(nativeQuery = true,
            value = "select zi.zone_id as \"zoneId\", zi.zone_name as \"zoneName\", zi.thumbnail, " +
                        "ui.display_name as \"displayName\", ui.head_img as \"headImg\", " +
                        "to_char(ar.apply_time, 'YYYY-MM-dd hh24:mi:ss') as \"applyTime\", " +
                        "ar.auth_status as \"authStatus\", ar.record_id as \"recorId\", " +
                        "aui.display_name as \"auditor\", ui.user_id as \"authorId\"," +
                        "to_char(ar.auth_time, 'YYYY-MM-dd hh24:mi:ss') as \"authTime\" " +
                    "from zone_info zi " +
                    "inner join audit_record ar on ar.zone_id = zi.zone_id " +
                    "inner join user_info ui on zi.user_id = ui.user_id " +
                    "left join user_info aui on ar.user_id = aui.user_id " +
                    "where ar.auth_status != 4 and ar.auth_status != 5 and delete_status is false " +
                        "and zone_name like concat('%', :name, '%') " +
                        "and (:searchType = 0 or (:searchType = 1 and ar.auth_status = 1 ) or (:searchType = 2 and ar.auth_status in (2, 3))) " +
                    "order by ar.apply_time desc"
    )
    Page<Map<String, Object>> auditCenter(@Param("name") String name,
                                          @Param("searchType") Integer searchType,
                                          Pageable pageable);

    @Modifying
    @Query(nativeQuery = true,
        value = "update zone_info set auth_status = :authStatus " +
                "where zone_id = (select zone_id from audit_record where record_id = :recordId)"
    )
    void updateAuthStatusWithAuditRecord(@Param("recordId") Integer recordId,
                                         @Param("authStatus") Integer authStatus);

    @Modifying
    @Query(nativeQuery = true,
        value = "update zone_info set auth_status = :authStatus where zone_id = :zoneId"
    )
    void updateAuthStatus(@Param("zoneId") Integer zoneId,
                          @Param("authStatus") Integer authStatus);

    @Modifying
    @Query(nativeQuery = true,
        value = "update zone_info set template_use = coalesce(template_use, 0) + 1 where zone_id = :zoneId"
    )
    void addTemplateUse(@Param("zoneId") Integer zoneId);

    @Modifying
    @Query(nativeQuery = true,
        value = "update zone_info set update_time = now() where zone_id = :zoneId"
    )
    void mergeUpdateTime(@Param("zoneId") Integer zoneId);

    @Query(nativeQuery = true,
        value = "select count(*) > 0 from zone_info where user_id is null and zone_name = :zoneName"
    )
    Boolean hasSameTemplateName(@Param("zoneName") String zoneName);

    @Query(nativeQuery = true,
            value = "select count(*) > 0 from zone_info " +
                    "where user_id is null and zone_name = :zoneName and zone_id != :zoneId"
    )
    Boolean hasSameTemplateNameWithoutOne(@Param("zoneName") String zoneName,
                                          @Param("zoneId") Integer zoneId);

    @Query(nativeQuery = true,
            value = "select zi.*, ui.display_name from zone_info zi, share_info si, user_info ui " +
                    "where zi.zone_id = si.zone_id and si.shared_user_id = :userId " +
                    "and si.accept_status = true " +
                    "and zi.zone_name like concat('%', :name, '%') " +
                    "and ui.user_id = zi.user_id " +
                    "order by zi.update_time desc"
    )
    Page<Map<String, Object>> pageQueryshareCenter(@Param("name") String name,
                                                   @Param("userId") Integer userId, Pageable pageable);


    @Query(nativeQuery = true,
            value = "select distinct(zone_name) as \"name\" from zone_info zi, share_info si " +
                    "where zi.zone_id = si.zone_id and si.shared_user_id = :userId and si.accept_status = true " +
                    "and zone_name like concat('%', :keyword, '%') " +
                    "order by zone_name"
    )
    List<String> autoCompleteShare(@Param("keyword") String keyword,
                                   @Param("userId") Integer userId);


    @Modifying
    @Query(nativeQuery = true,
            value = "update share_info set accept_status = :acceptStatus where zone_id = :zoneId and shared_user_id = :userId"
    )
    void acceptShare(@Param("zoneId") Integer zoneId,
                     @Param("userId") Integer userId,
                     @Param("acceptStatus") Boolean acceptStatus);
}
